/* Mptn -- regexp-like pattern matcher suited for morphological analysis
 * Copyright (C) 2000  George Bronnikov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
  Auxiliary functions.

  $Id: mptn-aux.c 1.2 Sun, 23 Jan 2000 20:56:51 -0600 goga $
*/

#include <string.h>
#include <glib.h>

#include "mptn-types.h"

/*
  Find a character contained in What, outside the brackets,
  where Leftb serve as left brackets, Rightb as right brackets,
  Esc serving as escape characters.
  (Exact balance of brackets is not counted; thus, [) would
  count as valid pair.

  Return a pointer to the character found, or NULL on failure.
*/
chr *
maux_find (chr *str, chr *what, chr *leftb, chr *rightb, chr *esc)
{
  int bcou;
  chr *ss;

  bcou = 0;

  for (ss = str; *ss; ss++)
    {
      if (bcou == 0 && chra_find (what, *ss))
	return str;
      else if (chra_find (leftb, *ss))
	bcou ++;
      else if (chra_find (rightb, *ss))
	{
	  bcou --;
	  
	  if (bcou < 0)
	    g_error ("More right brackets than left in %s", str);
	}
      else if (chra_find (esc, *ss))
	{
	  ss++;		/* will miss the next chr */
	  if (*ss == 0)	/* special case: that next chr is zero */
	    g_error ("Last chr escape chr in %s", str);
	}
    }

  /* Got to '\0', found nothing. */
  return NULL;
}
